/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2000 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: unicode.c,v 1.6 2000/12/12 18:14:13 tf Exp $
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#include "hana.h"

static unsigned int u2e[0xffff];

XChar2b u2xch(int ucs)
{
  XChar2b ch;
  int euc;

  euc = u2e[ucs];
  ch.byte1 = (euc >> 8) & 0x7f;
  ch.byte2 = euc & 0x7f;

  return ch;
}

int puteuc_string(const int *ucs)
{
  int i;
  
  for (i = 0; i < BUFLEN; i++) {
    if (ucs[i] == 0) {
      return 0;
    }
    if (ucs[i] < 0x100) {
      fputc(ucs[i], stderr);
    } else {
      fputc(u2e[ucs[i]] >> 8, stderr);
      fputc(u2e[ucs[i]] & 0xff, stderr);
    }
  }
  return 0;
}

int puteuc(int b1, int b2)
{
  int ucs;

  ucs = (b2 << 8) | b1;

  if (ucs < 0x100) {
    putchar(ucs);
  } else {
    putchar(u2e[ucs] >> 8);
    putchar(u2e[ucs] & 0xff);
  }
  return 0;
}

int make_u2e_table()
{
  int i, p;
  int fd;
  struct stat sb;
  unsigned char *addr;

  if ((fd = open(HANALIBDIR "/font.tbl", 0, O_RDONLY)) < 0) {
    fprintf(stderr, "\n***************************************************\n");
    fprintf(stderr, " Can't open %s/font.tbl file.\n", HANALIBDIR);
    fprintf(stderr, " Make sure to do 'make install' after compilation.\n");
    fprintf(stderr, "***************************************************\n");
    exit(1);
  }
  
  if (fstat(fd, &sb) < 0 ||
      (addr = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, fd, 0))
      == MAP_FAILED) {
    perror("mmap");
    exit(1);
  }
  close(fd);
  
  for (i = 0x391, p = 0; i < 0xffff; i++, p += 2) {
    u2e[i] = (addr[p] << 8) | addr[p + 1];
  }
  
  return 0;
}
